/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;
import net.sf.ext.util.Resources;

/**
 * A field for selecting a font. Available fonts are queried from the OS.
 * 
 * @author Robert Jan van der Waals
 */
public class FontSelector extends JComponent implements ISettingsComponent {
    
    private JLabel labelFont = ComponentFactory.getLabel(Resources.getText(DCLabels.lblFont));
    private JLabel labelFontSize = ComponentFactory.getLabel(Resources.getText(DCLabels.lblFontSize));
    private JLabel labelExample = ComponentFactory.getLabel(Resources.getText(DCLabels.lblPreview));
    private JComboBox comboFontName = ComponentFactory.getComboBox();
    private JComboBox comboFontSize = ComponentFactory.getComboBox();
    private LongTextField textField = ComponentFactory.getLongTextField(400);
    private JCheckBox checkBold = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblBold));
   
    /**
     * Initializes this field
     */
    public FontSelector() {
        buildComponent();
        fillFontNameCombo();
        fillFontSizeCombo();
    }
    
    /**
     * Fills the fonts combo box with values retrieved form the OS
     */
    private void fillFontNameCombo() {
        Font[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().
                       getAllFonts();
        
        for (int i = 0; i < fonts.length; i++) {
            comboFontName.addItem(fonts[i].getName());
        }
    }

    /**
     * Fills the font size combo
     */
    private void fillFontSizeCombo() {
        comboFontSize.addItem(new Integer(8));
        comboFontSize.addItem(new Integer(9));
        comboFontSize.addItem(new Integer(10));
        comboFontSize.addItem(new Integer(11));
        comboFontSize.addItem(new Integer(12));
        comboFontSize.addItem(new Integer(13));
        comboFontSize.addItem(new Integer(14));
        comboFontSize.addItem(new Integer(15));
        comboFontSize.addItem(new Integer(16));
        comboFontSize.addItem(new Integer(17));
        comboFontSize.addItem(new Integer(18));
    }
    
    /**
     * Returns the selected Font (with the chosen size, thickness)
     * Unless the user has chosen otherwise, Arial font size 11 is returned.
     */
    public Object getValue() {
        return getFont();
    }
    
    /**
     * Returns the selected Font (with the chosen size, thickness).
     * Unless the user has chosen otherwise, Arial font size 11 is returned.  
     */    
    public Font getFont() {
        if (comboFontName.getSelectedIndex() != -1) {
            String fontName = comboFontName.getSelectedItem().toString();
            
            int fontSize = 11;
            try {
            	fontSize= ((Integer) comboFontSize.getSelectedItem()).intValue();
            } catch (Exception exp) {}
            
            if (checkBold.isSelected()) {
                return new Font(fontName, Font.BOLD, fontSize);
            } else {
                return new Font(fontName, Font.PLAIN, fontSize);
            }
        } else {
        	return new Font("Arial", Font.PLAIN, 11);
        }
    }
    
    /**
     * Applies a value to this field
     */
    public void setValue(Object o) {
        Font font = (Font) o;
        try {
            checkBold.setSelected(font.isBold());
        	comboFontName.setSelectedItem(font.getName());
            comboFontSize.setSelectedItem(new Integer(font.getSize()));
        } catch (Exception ignore) {}
    }
    
    /**
     * Applies the selected values on a sample text
     */
    private void setSampleText() {
        textField.setFont(getFont());
    }

    /**
     * Builds this component
     */
    private void buildComponent() {
        setLayout(Layout.getGBL());
        comboFontName.addActionListener(new IndexHasChangedAction());
        comboFontSize.addActionListener(new IndexHasChangedAction());
        checkBold.addActionListener(new IndexHasChangedAction());
        
        textField.setBorder(ComponentFactory.getTitleBorder(""));
        textField.setText("data crow\r\nDATA CROW\r\nData Crow\r\ndATA cROW");
        
        JScrollPane scrollPane = new JScrollPane(textField);
        scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        
        add(labelFont,       Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
						     new Insets( 0, 0, 0, 0), 0, 0));
        add(comboFontName,   Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
    		                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
							 new Insets( 0, 0, 0, 0), 0, 0));
        add(labelFontSize,   Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 0, 0, 0), 0, 0));
        add(comboFontSize,   Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
        		            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
						     new Insets( 0, 0, 0, 0), 0, 0));
        add(checkBold,       Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
							 new Insets( 0, 0, 0, 0), 0, 0));
        add(labelExample,    Layout.getGBC( 0, 3, 2, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 0, 0, 0), 0, 0));
        add(scrollPane,      Layout.getGBC( 0, 4, 2, 1, 10.0, 10.0
        		            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
							 new Insets( 0, 0, 0, 0), 0, 0));
    }
    
    private class IndexHasChangedAction implements ActionListener {  
        public void actionPerformed(ActionEvent e) {
            setSampleText();
        }
    }
}
