/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.IconCache;

/**
 * Used to show images and text in a combo box
 * 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class ComboBoxRenderer extends JLabel implements ListCellRenderer {
    
    public ComboBoxRenderer() {
        setOpaque(true);
        setHorizontalAlignment(LEFT);
        setVerticalAlignment(CENTER);
    }

    public Component getListCellRendererComponent(
                                       JList list,
                                       Object value,
                                       int index,
                                       boolean isSelected,
                                       boolean cellHasFocus) {
        
        if (isSelected) {
            // if an item in the combo box is selected, change
            // the color of the back and foreground to indicate this
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
        } else {
            setBackground(list.getBackground());
            setForeground(list.getForeground());
        } 

        if (value instanceof DCObject) {
            // apply the text and the icon of the Data Crow object
            DCObject o = (DCObject) value;
            setIcon(IconCache.getIconForId(o.getID()));
            setText(o.toString());
        } else if (value instanceof DCModule) {
        	DCModule module = (DCModule) value;
            setIcon(module.getIcon());
            setText(module.toString());
        } else if (value != null){
            // set the icon to null for all other items
            setIcon(null);
            setText(value.toString());
        }
        
        Dimension preferred = getPreferredSize();
        
        preferred.height = 18;
        preferred.width = 100;
        setPreferredSize(preferred);
        
        return this;
    }
}
