/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.software;

import java.net.URL;

import javax.swing.ImageIcon;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Utilities;

/**
 * Representation of a Software Catagory.
 * Allows saving, updating and querying of Catagory items
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class SoftwareCategory extends DCObject {

    public static final int _A_NAME = 1;
    public static final int _B_ICON = 2;

    public SoftwareCategory() {
        super();
    }

    /**
     * Sets the module index / sets the ownership of this
     * data crow object  
     */      
    protected void setModuleIndex() {
        module = DCModules._SOFTWARECAT;
    }     

    /**
     * Returns the icon for this object
     */  
    public ImageIcon getIcon() {
        Object o = getValue(_B_ICON);
        if (o != null && o.toString().length() > 1) {
            try {
            	URL url = Utilities.createCachedImage(o.toString());
            	ImageIcon icon = new ImageIcon(url);
            	url = null;
            	return icon;
            } catch (Exception ignore) {}
        } else {
            return null;
        }
        return null;
    } 
    
    /**
     * Initialize all user settings on this data crow object 
     */  
    protected void initializeSettings() {
        DCFieldDefinition[] fields = getFields();
        
        int[] requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stSoftwareCatRequiredFields);
                                  
        for (int i = 0; i < requiredFields.length; i++) {
            fields[requiredFields[i]].setRequired(true);
        }
                                  
        int[] hiddenFields = Settings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsSoftwareCat);   

        for (int i = 0; i < hiddenFields.length; i++) {
            fields[hiddenFields[i]].setHidden(true);
        }        
    }

    /**
     * Create a UI representation for this object
     */
    public String toString() {
        return getValue(SoftwareCategory._A_NAME).toString();
    }    
}
