/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;

import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.movie.MovieFile;
import net.sf.dc.messages.Messages;
import net.sf.ext.mediamanager.FilePropertiesMovie;
import net.sf.ext.util.Utilities;

/**
 * Extract information from movie files (ogg, avi, mpeg, divx, xvid)
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class ImportMovieCD  extends ImportCDProcess {

    /**
     * Initializes (but does  not run) the Movie CD import process
     * @param ui used for displaying the progress
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportMovieCD(  ImportCDDialog ui,
                           String sourceDir,
                           boolean confirmDirs,
                           boolean recursiveDirs,
                           String cdName) {
        
        super(ui, sourceDir, confirmDirs, recursiveDirs, cdName);
    }

    public String[] getExtensions() {
    	return new String[] {"mpg", 
                             "mpeg", 
                             "riff", 
                             "avi", 
                             "vob", 
                             "ogm", 
                             "mov", 
                             "ifo", 
                             "mkv",
                             "asf"};
    }
    
    protected int getModule() {
        return DCModules._MOVIE;
    }

    /**
     * Parses the source file for information. 
     */    
    protected DCObject parseInformation(String filename) {
        Movie movie = new Movie();
        try {
            FilePropertiesMovie movieFile1 = new FilePropertiesMovie(filename);
            Integer playlength = new Integer(movieFile1.getDuration());
            
            movie.setValue(Movie._A_TITLE, getName(filename));
            movie.setValue(Movie._K_FILENAME, filename);
            
            if (playlength.intValue() <= 0 && movieFile1.getVideoWidth() <= 0) {
                return movie;
            }
            
            movie.setValue(Movie._J_PLAYLENGTH, playlength); 
            movie.setValue(Movie._L_VIDEOCODEC, movieFile1.getVideoCodec());
            movie.setValue(Movie._M_AUDIOCODEC, movieFile1.getAudioCodec());
            movie.setValue(Movie._N_WIDTH, new Integer(movieFile1.getVideoWidth()));
            movie.setValue(Movie._O_HEIGHT, new Integer(movieFile1.getVideoHeight()));
            movie.setValue(Movie._P_FPS, new Integer(movieFile1.getVideoRate()));
            movie.setValue(Movie._Q_FRAMES, new Integer(movieFile1.getDuration() * movieFile1.getVideoRate()));
            movie.setValue(Movie._R_AUDIOBITRATE, new Integer(movieFile1.getAudioBitRate()));
            movie.setValue(Movie._S_AUDIOSAMPLINGRATE, new Integer(movieFile1.getAudioRate()));
            movie.setValue(Movie._T_AUDIOCHANNEL, new Integer(movieFile1.getAudioChannels()));
            movie.setValue(Movie._0_SUBTITLELANGUAGE, movieFile1.getSubtitles());
            
            movie.setValue(Movie._2_FILESIZE, new Integer(getFileSize(filename)));
            
            int bitrate = movieFile1.getVideoBitRate();
            if (bitrate <= 0) {
                try {
                    MovieFile movieFile2 = new MovieFile(filename);
                    bitrate = new Double(movieFile2.getVideoBitrate()).intValue();
                } catch (Exception ignore) {}
            }
            
            movie.setValue(Movie._U_VIDEOBITRATE, new Integer(bitrate));
            movie.setValue(Movie._3_LOCATION, cdName);
            movie.setValue(Movie._0_SUBTITLELANGUAGE, Utilities.fillSubtitles(filename, ""));
            
            return movie;
        } catch (Exception exp) {
            Messages.add(exp);
            return null;
        }
    }
}
