/*
 * Element.java
 *
 * Created on November 19, 2002, 9:11 PM
 */

package net.sf.ext.mediamanager.ebml;

/**
     * Defines the basic EBML element.  Subclasses may provide child element access.
 * @author  John Cannon
 */
public class BinaryElement extends Element {

  /*private byte[] type = {
      0x00};*/
  private static int MIN_SIZE_LENGTH = 4;
  //private long size = 0;
  //protected byte[] data;

  /*
   * Creates a new instance of Element
   @param type The type ID of this element
   */
  public BinaryElement(byte[] type) {
    super(type);
  }

  /** Getter for property data.
   * @return Value of property data.
   *
   */
  public byte[] getData() {
    return this.data;
  }

  /** Setter for property data.
   * @param data New value of property data.
   *
   */
  public void setData(byte[] data) {
    this.data = data;
    this.size = data.length;
  }

  /** Getter for property size.
   * @return Value of property size.
   *
   */
  public long getSize() {
    return size;
  }

  /** Setter for property size.
   * @param size New value of property size.
   *
   */
  public void setSize(long size) {
    this.size = size;
  }

  /** Getter for property type.
   * @return Value of property type.
   *
   */
  public byte[] getType() {
    return type;
  }

  /** Setter for property type.
   * @param type New value of property type.
   *
   */
  public void setType(byte[] type) {
    this.type = type;
  }

  public byte[] toByteArray() {
    byte[] head = makeEbmlCode(type, size);
    byte[] ret = new byte[head.length + data.length];
    System.arraycopy(head, 0, ret, 0, head.length);
    System.arraycopy(data, 0, ret, head.length, data.length);
    return ret;
  }

  public static void setMinSizeLength(int minSize) {
    MIN_SIZE_LENGTH = minSize;
  }

  public static int getMinSizeLength() {
    return MIN_SIZE_LENGTH;
  }

  public static byte[] makeEbmlCode(byte[] typeID, long size) {
    int codedLen = codedSizeLength(size);
    byte[] ret = new byte[typeID.length + codedLen];
    System.arraycopy(typeID, 0, ret, 0, typeID.length);
    byte[] codedSize = makeEbmlCodedSize(size);
    System.arraycopy(codedSize, 0, ret, typeID.length, codedSize.length);
    return ret;
  }

  public static byte[] makeEbmlCodedSize(long size) {
    int len = codedSizeLength(size);
    byte[] ret = new byte[len];
    //System.out.println("Coding " + size + " in " + len + "bytes...");
    //byte[] packedSize = packIntUnsigned(size);
    long mask = 0x00000000000000FFL;
    for (int i = 0; i < len; i++) {
      ret[len - 1 - i] = (byte)((size & mask) >>> (i * 8));
      //System.out.print(" " + Integer.toHexString(ret[len - 1 - i]) + " ");
      mask <<= 8;
    }
    //System.out.print("\n");
    //The first size bits should be clear, otherwise we have an error in the size determination.
    ret[0] |= (0x80 >> (len - 1));
    return ret;
  }

  public static int getMinByteSize(long value) {
    if (value <= 127 && value >= (0 - 0x0080)) {
      return 1;
    }
    else if (value <= 0x007FFF && value >= (0 - 0x008000)) {
      return 2;
    }
    else if (value <= 0x007FFFFF && value >= (0 - 0x00800000L)) {
      return 3;
    }
    else if (value <= 0x007FFFFFFF && value >= (0 - 0x0080000000L)) {
      return 4;
    }
    else if (value <= 0x007FFFFFFFFFL && value >= (0 - 0x008000000000L)) {
      return 5;
    }
    else if (value <= 0x007FFFFFFFFFFFL && value >= (0 - 0x00800000000000L)) {
      return 6;
    }
    else if (value <= 0x007FFFFFFFFFFFFFL && value >= (0 - 0x80000000000000L)) {
      return 7;
    }
    else {
      return 8;
    }
    /* long mask = 0xFF00000000000000L;
             for(int i = 0; i < 8; i++){
        if((value & mask) == mask){
            //Skip
        }else if((value & mask) == 0x0000000000000000L){
            //Skip
        }else{
         if((i == 0) && ((value & 0x8000000000000000L) == 0x8000000000000000L)){
                //Skip
            }else{
                return (8-i);
            }
        }
        mask >>>= 8;
             }
             return 8;*/
  }

  public static int getMinByteSizeUnsigned(long value) {
    int size = 8;
    long mask = 0xFF00000000000000L;
    for (int i = 0; i < 7; i++) {
      if ((value & mask) == 0) {
        mask >>>= 8;
        size--;
      }
      else {
        return size;
      }
    }
    return 8;
  }

  public static int codedSizeLength(long value) {
    int codedSize = 0;
    if (value < 127) {
      codedSize = 1;
    }
    else if (value < 16383) {
      codedSize = 2;
    }
    else if (value < 2097151) {
      codedSize = 3;
    }
    else if (value < 268435455) {
      codedSize = 4;
    }
    if ((MIN_SIZE_LENGTH > 0) && (codedSize <= MIN_SIZE_LENGTH)) {
      codedSize = MIN_SIZE_LENGTH;
    }
    else {
      codedSize = 8;
    }
    return codedSize;
  }

  public static byte[] packIntUnsigned(long value) {
    int size = getMinByteSizeUnsigned(value);
    byte[] ret = new byte[size];
    long mask = 0x00000000000000FFL;
    for (int i = 0; i < size; i++) {
      ret[size - 1 - i] = (byte)((value & mask) >>> (8 * i));
      mask <<= 8;
    }
    return ret;
  }

  public static byte[] packInt(long value) {
    int size = getMinByteSize(value);
    byte[] ret = new byte[size];
    long mask = 0x00000000000000FFL;
    for (int i = 0; i < size; i++) {
      ret[size - 1 - i] = (byte)((value & mask) >>> (8 * i));
      mask <<= 8;
    }

    /*if(value < 0){
        ret[0] |= 0x80; //Preserve Sign bit
             }*/
    return ret;
  }
}
